{% extends "organizations_base.html" %}
{% block content %}
{% if profile %}
<h1>Update your profile</h1>
{% else %}
<h1> {{ organization_user }} @ {{ organization }}</h1>
{% endif %}
{% if user == organization_user.user %}This is you!{% endif %}
<form action="." method="post">{% csrf_token %}
    {{ form }}
    <input type="submit" value="submit">
</form>
{% endblock %}
