{% extends "organizations_base.html" %}
{% block content %}
<h1>{{ organization }}</h1>
<form action="." method="post">{% csrf_token %}
    {{ form }}
    <input type="submit" value="submit">
</form>
{% endblock %}
