{% extends "organizations_base.html" %}
{% load url from future %}
{% load org_tags %}
{% block content %}
<h1>{{ organization }}</h1>
<ul>
    <li><a href="{% url "organization_edit" organization.pk %}">Edit</a></li>
    <li><a href="{% url "organization_delete" organization.pk %}">Delete</a></li>
    <li><a href="{% url "organization_user_add" organization.pk %}">Add a member</a></li>
</ul>
{% organization_users organization %}
{% endblock %}

